﻿using System;
using System.Data;
using System.Runtime.InteropServices;

namespace SQLiteWrapper
{
    public class SQLiteException : Exception
    {
        public SQLiteException(string message) :
            base(message)
        { }
    }

    public static class SQLiteNative
    {
        public const int SQLITE_OK = 0;
        public const int SQLITE_ROW = 100;
        public const int SQLITE_DONE = 101;
        public const int SQLITE_INTEGER = 1;
        public const int SQLITE_FLOAT = 2;
        public const int SQLITE_TEXT = 3;
        public const int SQLITE_BLOB = 4;
        public const int SQLITE_NULL = 5;

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_open")]
        public static extern int sqlite3_open(string filename, out IntPtr db);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_close")]
        public static extern int sqlite3_close(IntPtr db);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_prepare_v2")]
        public static extern int sqlite3_prepare_v2(IntPtr db, string zSql,
            int nByte, out IntPtr ppStmpt, IntPtr pzTail);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_step")]
        public static extern int sqlite3_step(IntPtr stmHandle);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_finalize")]
        public static extern int sqlite3_finalize(IntPtr stmHandle);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_errmsg")]
        public static extern string sqlite3_errmsg(IntPtr db);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_column_count")]
        public static extern int sqlite3_column_count(IntPtr stmHandle);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_column_origin_name")]
        public static extern string sqlite3_column_origin_name(
            IntPtr stmHandle, int iCol);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_column_type")]
        public static extern int sqlite3_column_type(IntPtr stmHandle, int iCol);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_column_int")]
        public static extern int sqlite3_column_int(IntPtr stmHandle, int iCol);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_column_text")]
        public static extern string sqlite3_column_text(IntPtr stmHandle, int iCol);

        [DllImport("sqlite3.dll", EntryPoint = "sqlite3_column_double")]
        public static extern double sqlite3_column_double(IntPtr stmHandle, int iCol);
    }

    public class SQLite : IDisposable
    {
        private IntPtr _db; //pointer to SQLite database
        private bool _open; //whether or not the database is open

        public SQLite(string path)
        {
            OpenDatabase(path);
        }

        /// <summary>
        /// Opens or creates SQLite database with the specified path
        /// </summary>
        /// <param name="path">Path to SQLite database</param>
        public void OpenDatabase(string path)
        {
            if (SQLiteNative.sqlite3_open(path, out _db) != SQLiteNative.SQLITE_OK)
                throw new SQLiteException("Could not open database file: " + path);

            _open = true;
        }

        /// <summary>
        /// Closes the SQLite database
        /// </summary>
        public void CloseDatabase()
        {
            if (_open)
                SQLiteNative.sqlite3_close(_db);

            _open = false;
        }

        /// <summary>
        /// Executes a query that returns no results
        /// </summary>
        /// <param name="query">SQL query to execute</param>
        public void ExecuteNonQuery(string query)
        {
            if (!_open)
                throw new SQLiteException("SQLite database is not open.");

            //prepare the statement
            IntPtr stmHandle = Prepare(query);

            if (SQLiteNative.sqlite3_step(stmHandle) != SQLiteNative.SQLITE_DONE)
                throw new SQLiteException("Could not execute SQL statement.");

            Finalize(stmHandle);
        }

        /// <summary>
        /// Executes a query and stores the results in
        /// a DataTable
        /// </summary>
        /// <param name="query">SQL query to execute</param>
        /// <returns>DataTable of results</returns>
        public DataTable ExecuteQuery(string query)
        {
            if (!_open)
                throw new SQLiteException("SQLite database is not open.");

            //prepare the statement
            IntPtr stmHandle = Prepare(query);

            //get the number of returned columns
            int columnCount = SQLiteNative.sqlite3_column_count(stmHandle);

            //create datatable and columns
            DataTable dTable = new DataTable();
            for (int i = 0; i < columnCount; i++)
                dTable.Columns.Add(SQLiteNative.sqlite3_column_origin_name(stmHandle, i));

            //populate datatable
            while (SQLiteNative.sqlite3_step(stmHandle) == SQLiteNative.SQLITE_ROW)
            {
                object[] row = new object[columnCount];
                for (int i = 0; i < columnCount; i++)
                {
                    switch (SQLiteNative.sqlite3_column_type(stmHandle, i))
                    {
                        case SQLiteNative.SQLITE_INTEGER:
                            row[i] = SQLiteNative.sqlite3_column_int(stmHandle, i);
                            break;
                        case SQLiteNative.SQLITE_TEXT:
                            row[i] = SQLiteNative.sqlite3_column_text(stmHandle, i);
                            break;
                        case SQLiteNative.SQLITE_FLOAT:
                            row[i] = SQLiteNative.sqlite3_column_double(stmHandle, i);
                            break;
                    }
                }

                dTable.Rows.Add(row);
            }

            Finalize(stmHandle);

            return dTable;
        }

        /// <summary>
        /// Prepares a SQL statement for execution
        /// </summary>
        /// <param name="query">SQL query</param>
        /// <returns>Pointer to SQLite prepared statement</returns>
        private IntPtr Prepare(string query)
        {
            IntPtr stmHandle;

            if (SQLiteNative.sqlite3_prepare_v2(_db, query, query.Length,
                  out stmHandle, IntPtr.Zero) != SQLiteNative.SQLITE_OK)
                throw new SQLiteException(SQLiteNative.sqlite3_errmsg(_db));

            return stmHandle;
        }

        /// <summary>
        /// Cleans up a SQLite database connection
        /// </summary>
        public void Dispose()
        {
            if (_open)
                this.CloseDatabase();
        }

        /// <summary>
        /// Finalizes a SQLite statement
        /// </summary>
        /// <param name="stmHandle">
        /// Pointer to SQLite prepared statement
        /// </param>
        private void Finalize(IntPtr stmHandle)
        {
            if (SQLiteNative.sqlite3_finalize(stmHandle) != SQLiteNative.SQLITE_OK)
                throw new SQLiteException("Could not finalize SQL statement.");
        }

    }
}

