﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SQLite;
using System.Linq;
using System.Text;

namespace ConsoleApp
{
    using SQLiteWrapper;
    using Community.CsharpSqlite.SQLiteClient;

    class Program
    {
        static void Main(string[] args)
        {
            // If the database file already exists, then this is the second-plus
            // time we've run the code; blow it away so everything builds on top
            // of each other successfully. (Otherwise the CREATE TABLE statement
            // in the NativeMain() call will blow chunks.)
            if (System.IO.File.Exists(@"persons.sqlite"))
                System.IO.File.Delete(@"persons.sqlite");

            Console.WriteLine("Native calls:");
            NativeMain();
            Console.WriteLine();

            Console.WriteLine("Native calls wrapped:");
            NativeWrapperMain();
            Console.WriteLine();

            Console.WriteLine("Managed wrapper calls:");
            ManagedWrapperMain();
            Console.WriteLine();

            Console.WriteLine("All-managed calls:");
            AllManagedMain();
            Console.WriteLine();
        }

        static void NativeMain()
        {
            // Open the database--db is our "handle" to it
            IntPtr db;
            if (SQLiteNative.sqlite3_open(@"persons.sqlite", out db) 
                == SQLiteNative.SQLITE_OK)
            {
                // Prepare a simple DDL "CREATE TABLE" statement
                string query = 
                    "CREATE TABLE Persons " + 
                    "(first TEXT, last TEXT, age INTEGER)";
                IntPtr stmHandle;
                if (SQLiteNative.sqlite3_prepare_v2(db, query, query.Length,
                      out stmHandle, IntPtr.Zero) != SQLiteNative.SQLITE_OK)
                {
                    // Something went wrong--find out what
                    var err = SQLiteNative.sqlite3_errmsg(db);
                }
                if (SQLiteNative.sqlite3_step(stmHandle) != SQLiteNative.SQLITE_DONE)
                {
                    // Something went wrong--find out what
                    var err = SQLiteNative.sqlite3_errmsg(db);
                }
                if (SQLiteNative.sqlite3_finalize(stmHandle) != SQLiteNative.SQLITE_OK)
                {
                    // Something went wrong--find out what
                    var err = SQLiteNative.sqlite3_errmsg(db);
                }

                // ... Now that we've created a table, we can insert some
                // data, query it back, and so on

                // Close the database back up
                SQLiteNative.sqlite3_close(db);
            }
        }

        static void NativeWrapperMain()
        {
            using (SQLite db = new SQLite("persons.sqlite"))
            {
                db.ExecuteNonQuery("INSERT INTO Persons (first, last, age) " +
                    "VALUES ('Aaron', 'Erickson', 38)");
                db.ExecuteNonQuery("INSERT INTO Persons (first, last, age) " +
                    "VALUES ('Rick', 'Minerich', 29)");
                db.ExecuteNonQuery("INSERT INTO Persons (first, last, age) " +
                    "VALUES ('Talbott', 'Crowell', 35)");

                DataTable table = db.ExecuteQuery("SELECT * FROM Persons");

                foreach (DataRow row in table.Rows)
                {
                    Console.WriteLine("{0} {1} {2}", row[0], row[1], row[2]);
                }
            }
        }
        static void ManagedWrapperMain()
        {
            var connStr = new SQLiteConnectionStringBuilder() { DataSource = "persons.sqlite" };
            using (SQLiteConnection conn = new SQLiteConnection(connStr.ToString()))
            {
                conn.Open();
                SQLiteCommand cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT COUNT(*) FROM Persons";
                var ct = cmd.ExecuteScalar();
                Console.WriteLine("Count = {0}", ct);

                cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT * FROM Persons";
                SQLiteDataReader reader = cmd.ExecuteReader();
                DataTable dt = new DataTable();
                dt.Load(reader);
                foreach (DataRow row in dt.Rows)
                {
                    Console.WriteLine("{0} {1} {2}", row[0], row[1], row[2]);
                }
            }
        }
        static void AllManagedMain()
        {
            SqliteConnection conn = new SqliteConnection(@"Version=3,uri=file:persons.sqlite");
            conn.Open();
            try
            {
                SqliteCommand cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT COUNT(*) FROM Persons";
                var ct = cmd.ExecuteScalar();
                Console.WriteLine("Count = {0}", ct);

                cmd = conn.CreateCommand();
                cmd.CommandText = "SELECT * FROM Persons";
                SqliteDataReader reader = cmd.ExecuteReader();
                while (reader.Read())
                {
                    Console.WriteLine("{0} {1} {2}", reader[0], reader[1], reader[2]);
                }
            }
            finally
            {
                conn.Close();
            }
        }
    }
}
